
import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:flutter/widgets.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class ChobiAdMobAdaptiveBanner extends StatefulHookConsumerWidget {

  /*
  late final String lastClickedTimePrefsKey;

  final String? androidBannerId;
  final String? iosBannerId;

  final bool isDebugMode;

  final void Function(ConsentStatus status)? onConsentChecked;
  final void Function(FormError err)? onError;

   */

  final ChobiAdMob chobiAdMob;

  ChobiAdMobAdaptiveBanner({
    Key? key,
    required this.chobiAdMob,
  });

  @override
  ConsumerState<ChobiAdMobAdaptiveBanner> createState() => _ChobiAdMobAdaptiveBannerState();
}

class _ChobiAdMobAdaptiveBannerState extends ConsumerState<ChobiAdMobAdaptiveBanner> {

  @protected
  final isAdAvailableProvider = StateProvider((ref) => false);

  @protected
  final isAdLoadedProvider = StateProvider((ref) => false);

  final _bannerAd = StateProvider<BannerAd?>((ref) => null);

  @protected
  ChobiAdMob get chobiAdMob => widget.chobiAdMob;


  /*
  @override
  List<String> get testIds => [
    '076A22BB69BB7E316A069C4E08F48036',
  ];

   */

  /*
  @override
  void onConsentCheckError(FormError error) {
    widget.onError?.call(error);
  }

   */

  /*
  @override
  void onConsentCheckFinished(ConsentStatus status) {
    widget.onConsentChecked?.call(status);
    if (status != ConsentStatus.required) {
      loadBanner();
    }
  }

   */



  @protected
  Future<void> loadBanner() async {

    if (!ref.read(isAdAvailableProvider)) {
      debugPrint('AD not available');
      return;
    }

    final bannerId = chobiAdMob.getAdId();

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);

    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );
    ref.read(_bannerAd.notifier).state = ad;
    await ad.load();
  }


  @protected
  Future<void> disposeBanner() async {
    final ad = ref.read(_bannerAd);
    if (ad != null) {
      await ad.dispose();
      ref.read(_bannerAd.notifier).state = null;
    }
  }

  /*
  @protected
  Future<void> startBannerProcess() async {
    debugPrint('start banner process');

    await startAdProcess(
      onBeforeStartProcess: () async {
        await disposeBanner();
      }
    );
  }

   */

  void _onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      ref.read(isAdAvailableProvider.notifier).state = true;
    }
  }

  void _onConsentError(FormError error) {
    ref.read(isAdAvailableProvider.notifier).state = false;
    debugPrint('$error');
  }

  @override
  void initState() {
    super.initState();

    debugPrint('banner initState');

    chobiAdMob.onConsentCheckFinished = _onConsentCheckFinished;
    chobiAdMob.onConsentCheckError = _onConsentError;

    chobiAdMob.checkPermissionAndStartAdProcess();
  }

  @override
  void dispose() {
    Future.sync(() async {
      await chobiAdMob.stopAdProcess();
      await disposeBanner();
    });

    super.dispose();
  }


  final _blankBox = const SizedBox.shrink();

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
      onAdLoaded: (ad) {
        ref.read(isAdLoadedProvider.notifier).state = true;
      },
      onAdFailedToLoad: (ad, err) {
        debugPrint('admob banner laod error: ${err.message}');
        ref.read(isAdLoadedProvider.notifier).state = false;
        disposeBanner();
      },
      onAdClicked: (ad) {
        chobiAdMob.updateLastClickedTime();
        chobiAdMob.checkPermissionAndStartAdProcess();
      }
    );
  }

  @override
  Widget build(BuildContext context) {
    //final isAdOk = ref.watch(isAdAvailableProvider);
    final ad = ref.watch(_bannerAd);
    
    if (ad != null) {
      return Container(
        width: double.infinity,
        height: ad.size.height.toDouble(),
        alignment: Alignment.center,
        child: AdWidget(ad: ad),
      );
    } else {
      return _blankBox;
    }
  }

}
